@@ -33,6 +33,7 @@
 
 package provide portdestroot 1.0
 package require portutil 1.0
+package require fileutil
 
 set org.macports.destroot [target_new org.macports.destroot portdestroot::destroot_main]
 target_provides ${org.macports.destroot} destroot
@@ -61,25 +62,26 @@ default destroot.destdir {DESTDIR=${destroot}}
 default destroot.nice {${buildnicevalue}}
 default destroot.umask {$system_options(destroot_umask)}
 default destroot.clean no
-default destroot.keepdirs ""
+default destroot.keepdirs {}
 default destroot.violate_mtree no
 default destroot.delete_la_files {${delete_la_files}}
 
 set_ui_prefix
 
 proc portdestroot::destroot_getargs {args} {
-    if {(([option build.type] eq "default" && [option os.platform] ne "freebsd") || \
-         ([option build.type] eq "gnu")) \
-        && [regexp "^(/\\S+/|)(g|gnu|)make(\\s+.*|)$" [option destroot.cmd]]} {
+    global build.type os.platform destroot.cmd destroot.target
+    if {((${build.type} eq "default" && ${os.platform} ne "freebsd") ||
+         (${build.type} eq "gnu"))
+        && [regexp "^(/\\S+/|)(g|gnu|)make(\\s+.*|)$" ${destroot.cmd}]} {
         # Print "Entering directory" lines for better log debugging
-        return "-w [option destroot.target]"
+        return "-w ${destroot.target}"
     }
 
-    return "[option destroot.target]"
+    return ${destroot.target}
 }
 
 proc portdestroot::destroot_start {args} {
-    global UI_PREFIX prefix subport porturl destroot os.platform destroot.clean portsharepath \
+    global UI_PREFIX prefix subport destroot os.platform destroot.clean portsharepath \
            destroot.umask destroot.asroot euid egid \
            applications_dir frameworks_dir
     variable oldmask
@@ -112,28 +114,27 @@ proc portdestroot::destroot_start {args} {
 
     file mkdir "${destroot}"
     if { ${os.platform} eq "darwin" } {
-        system -W ${destroot} "${mtree} -e -U -f [file join ${portsharepath} install macosx.mtree]"
+        system -W ${destroot} "${mtree} -e -U -f [shellescape [file join ${portsharepath} install macosx.mtree]]"
         file mkdir "${destroot}${applications_dir}"
         file mkdir "${destroot}${frameworks_dir}"
     }
     file mkdir "${destroot}${prefix}"
-    system -W ${destroot}${prefix} "${mtree} -e -U -f [file join ${portsharepath} install prefix.mtree]"
+    system -W ${destroot}${prefix} "${mtree} -e -U -f [shellescape [file join ${portsharepath} install prefix.mtree]]"
+
+    # Create startup-scripts/items
+    portstartupitem::startupitem_create
 }
 
 proc portdestroot::destroot_main {args} {
-    command_exec destroot
+    command_exec -callback portprogress::target_progress_callback destroot
     return 0
 }
 
 proc portdestroot::destroot_finish {args} {
     global UI_PREFIX destroot prefix subport destroot.violate_mtree \
-           applications_dir frameworks_dir destroot.keepdirs destroot.delete_la_files \
-           os.platform os.version
+           applications_dir frameworks_dir destroot.keepdirs destroot.delete_la_files
     variable oldmask
 
-    # Create startup-scripts/items
-    portstartupitem::startupitem_create
-
     foreach fileToDelete {share/info/dir lib/charset.alias} {
         if {[file exists "${destroot}${prefix}/${fileToDelete}"]} {
             ui_debug "Deleting stray ${fileToDelete} file."
@@ -144,7 +145,7 @@ proc portdestroot::destroot_finish {args} {
     # Prevent overlinking due to glibtool .la files: https://trac.macports.org/ticket/38010
     ui_debug "Fixing glibtool .la files in destroot for ${subport}"
     set la_file_list [list]
-    fs-traverse -depth fullpath ${destroot} {
+    fs-traverse -depth fullpath [list $destroot] {
         if {[file extension $fullpath] eq ".la" && ([file type $fullpath] eq "file" || [file type $fullpath] eq "link")} {
             if {[file type $fullpath] eq "link" && [file pathtype [file link $fullpath]] ne "relative"} {
                 # prepend $destroot to target of absolute symlinks
@@ -175,6 +176,8 @@ proc portdestroot::destroot_finish {args} {
 
     # Prune empty directories in ${destroot}
     foreach path ${destroot.keepdirs} {
+        # Prepend $destroot if $path isn't already inside it
+        set path [::fileutil::jail $destroot $path]
         if {![file isdirectory ${path}]} {
             xinstall -m 0755 -d ${path}
         }
@@ -182,7 +185,7 @@ proc portdestroot::destroot_finish {args} {
             xinstall -c -m 0644 /dev/null ${path}/.turd_${subport}
         }
     }
-    fs-traverse -depth dir ${destroot} {
+    fs-traverse -depth dir [list ${destroot}] {
         if {[file type $dir] eq "directory"} {
             catch {file delete $dir}
         }
@@ -199,12 +202,14 @@ proc portdestroot::destroot_finish {args} {
     }
 
     # Compress all manpages with gzip (instead)
-    set manpath "${destroot}${prefix}/share/man"
-    set gzip [findBinary gzip ${portutil::autoconf::gzip_path}]
-    set gunzip "$gzip -d"
-    set bunzip2 "[findBinary bzip2 ${portutil::autoconf::bzip2_path}] -d"
+    set manpath ${destroot}${prefix}/share/man
     if {[file isdirectory ${manpath}] && [file type ${manpath}] eq "directory"} {
         ui_info "$UI_PREFIX [format [msgcat::mc "Compressing man pages for %s"] ${subport}]"
+
+        set gzip [findBinary gzip ${portutil::autoconf::gzip_path}]
+        set gunzip "$gzip -d"
+        set bunzip2 "[findBinary bzip2 ${portutil::autoconf::bzip2_path}] -d"
+
         set found 0
         set manlinks [list]
         set mandir_re {^(cat|man)(.)$}
@@ -223,17 +228,17 @@ proc portdestroot::destroot_finish {args} {
                         if {[regexp ${gzfile_re} ${manfile} gzfile manfile]} {
                             set found 1
                             system -W ${manpath} \
-                            "$gunzip -f [file join ${mandir} ${gzfile}] && \
-                            $gzip -9vnf [file join ${mandir} ${manfile}]"
+                            "$gunzip -f [shellescape [file join ${mandir} ${gzfile}]] && \
+                            $gzip -9vnf [shellescape [file join ${mandir} ${manfile}]]"
                         } elseif {[regexp ${bz2file_re} ${manfile} bz2file manfile]} {
                             set found 1
                             system -W ${manpath} \
-                            "$bunzip2 -f [file join ${mandir} ${bz2file}] && \
-                            $gzip -9vnf [file join ${mandir} ${manfile}]"
+                            "$bunzip2 -f [shellescape [file join ${mandir} ${bz2file}]] && \
+                            $gzip -9vnf [shellescape [file join ${mandir} ${manfile}]]"
                         } elseif {[regexp ${normalfile_re} ${manfile}]} {
                             set found 1
                             system -W ${manpath} \
-                            "$gzip -9vnf [file join ${mandir} ${manfile}]"
+                            "$gzip -9vnf [shellescape [file join ${mandir} ${manfile}]]"
                         }
                         set gzmanfile ${manfile}.gz
                         set gzmanfilepath [file join ${mandirpath} ${gzmanfile}]
@@ -330,7 +335,7 @@ proc portdestroot::destroot_finish {args} {
                 } else {
                     set dir_allowed no
                     # these files are (at least potentially) outside of the prefix
-                    foreach dir "$applications_dir $frameworks_dir /Library/LaunchAgents /Library/LaunchDaemons /Library/StartupItems" {
+                    foreach dir [list $applications_dir $frameworks_dir /Library/LaunchAgents /Library/LaunchDaemons /Library/StartupItems] {
                         if {[string equal -length [expr {[string length $dfile] + 1}] $dfile/ $dir]} {
                             # it's a prefix of one of the allowed paths
                             set dir_allowed yes
@@ -358,12 +363,33 @@ proc portdestroot::destroot_finish {args} {
 
         # abort here only so all violations can be observed
         if { ${mtree_violation} ne "no" } {
-            ui_warn "[format [msgcat::mc "%s violates the layout of the ports-filesystems!"] [option subport]]"
+            ui_warn "[format [msgcat::mc "%s violates the layout of the ports-filesystems!"] $subport]"
             ui_warn "Please fix or indicate this misbehavior (if it is intended), it will be an error in future releases!"
             # error "mtree violation!"
         }
     } else {
-        ui_warn "[format [msgcat::mc "%s installs files outside the common directory structure."] [option subport]]"
+        ui_warn "[format [msgcat::mc "%s installs files outside the common directory structure."] $subport]"
+    }
+
+    # Work around apparent filesystem bug.
+    # https://trac.macports.org/ticket/67336
+    if {[fs_clone_capable $destroot]} {
+        global workpath
+        ui_debug "Applying sparse file lseek bug workaround"
+        try {
+            fs-traverse -depth fullpath [list $destroot] {
+                if {[file type $fullpath] eq "file" && [fileIsSparse $fullpath]} {
+                    ui_debug "Cloning $fullpath for workaround"
+                    clonefile $fullpath ${workpath}/.macports-sparse-workaround
+                    file delete ${workpath}/.macports-sparse-workaround
+                    if {![fileIsSparse $fullpath]} {
+                        ui_debug "$fullpath is no longer sparse"
+                    }
+                }
+            }
+        } on error {eMessage} {
+            ui_debug "Error while applying sparse file workaround: $eMessage"
+        }
     }
 
     # Restore umask
